#include <QTime>
#include <QDebug>
#include "Map.h"

CMap  map;

//---------------------------------------------------------------------------------------------
CWall::CWall()
{
    mType = 0;
}

//---------------------------------------------------------------------------------------------
CTile::CTile()
{
    mType = 0;
}

//---------------------------------------------------------------------------------------------
CMap::CMap()
{
    // default values
    mSize = CVec2(15, 10);
    mTiles = NULL;
}

//---------------------------------------------------------------------------------------------
CMap::~CMap()
{
    if (mTiles != NULL)
        delete[] mTiles;
}

//---------------------------------------------------------------------------------------------
void CMap::resize()
{
    if (mTiles != NULL)
        delete[] mTiles;

    mTiles = new CTile[mSize.x * mSize.y];
}

//---------------------------------------------------------------------------------------------
void   CMap::setTile(CVec2 pos, uint8_t type)
{
    if (pos.x >= 0 && pos.x < mSize.x &&
        pos.y >= 0 && pos.y < mSize.y)
    {
        getTile(pos)->mType = type;
    }
}

//---------------------------------------------------------------------------------------------
void   CMap::setWall(CVec2 pos, EWallSide side, uint8_t type)
{
    if (pos.x >= 0 && pos.x < mSize.x &&
        pos.y >= 0 && pos.y < mSize.y)
    {
        getTile(pos)->mWalls[side].mType = type;
    }
}

//---------------------------------------------------------------------------------------------
CTile*  CMap::getTile(CVec2 pos)
{
    if (pos.x < 0 || pos.x >= mSize.x ||
        pos.y < 0 || pos.y >= mSize.y)
        return NULL;

    return(&mTiles[pos.y * mSize.x + pos.x]);
}

//---------------------------------------------------------------------------------------------
bool CMap::isLoaded()
{
    return mTiles != NULL;
}

//---------------------------------------------------------------------------------------------
char    gFileHeader[] = "DMMP"; // Dungeon Master MaP
#define CURRENT_VERSION     1
#define CURRENT_REVISION    0

void CMap::save(char* fileName)
{
    FILE*       handle = fopen(fileName, "wb");
    uint16_t    ver = CURRENT_VERSION;
    uint16_t    rev = CURRENT_REVISION;

    if (handle != NULL)
    {
        // header, version, revision
        fwrite(gFileHeader, 1, 4, handle);
        fwrite(&ver, 2, 1, handle);
        fwrite(&rev, 2, 1, handle);

        // map size
        fwrite(&mSize.x, 4, 1, handle);
        fwrite(&mSize.y, 4, 1, handle);

        // map data
        for (int y = 0; y < mSize.y; ++y)
            for (int x = 0; x < mSize.x; ++x)
            {
                CTile*  t = getTile(CVec2(x, y));
                fwrite(&t->mType, 1, 1, handle);

                for (int side = 0; side < eWallSideMax; ++side)
                    fwrite(&t->mWalls[side].mType, 1, 1, handle);
            }

        fclose(handle);
    }
}

//---------------------------------------------------------------------------------------------
void CMap::load(char* fileName)
{
    FILE*       handle = fopen(fileName, "rb");
    char        header[4 + 1];
    uint16_t    ver;
    uint16_t    rev;

    if (handle != NULL)
    {
        // header, version, revision
        fread(header, 1, 4, handle);
        header[4] = 0;
        fread(&ver, 2, 1, handle);
        fread(&rev, 2, 1, handle);

        // check if it's the right version
        if (strcmp(header, gFileHeader) != 0 ||
            ver != CURRENT_VERSION ||
            rev != CURRENT_REVISION)
        {
            fclose(handle);
            return;
        }

        // map size
        fread(&mSize.x, 4, 1, handle);
        fread(&mSize.y, 4, 1, handle);

        // allocate tiles
        if (mTiles != NULL)
            delete[] mTiles;

        mTiles = new CTile[mSize.x * mSize.y];

        // map data
        for (int y = 0; y < mSize.y; ++y)
            for (int x = 0; x < mSize.x; ++x)
            {
                CTile*  t = getTile(CVec2(x, y));
                fread(&t->mType, 1, 1, handle);

                for (int side = 0; side < eWallSideMax; ++side)
                    fread(&t->mWalls[side].mType, 1, 1, handle);
            }

        fclose(handle);
    }
}
