#ifndef CBRIDGE_H
#define CBRIDGE_H

#include <QObject>
#include <QQuickImageProvider>
#include "../common/sources/system/cmaths.h"

// bridge between the QML and the C++ part

#define qmlMember(_type,_name) \
    public: \
        Q_PROPERTY(_type _name MEMBER m_##_name READ get_##_name WRITE set_##_name NOTIFY _name##Changed); \
        _type get_##_name() const {return m_##_name;} \
        void set_##_name(_type val) {m_##_name = val; emit _name##Changed(val);} \
    private: \
        _type m_##_name;

enum    ETabIndex
{
    eTabMap = 0,
    eTabTiles,
    eTabWalls,
    eTabObjects
};

enum    EQMLTools
{
    eQMLToolDraw = 0,
    eQMLToolSelect,
    eQMLToolPaste
};

//---------------------------------------------------------------------------------------------
class CBridge : public QObject
{
    Q_OBJECT

public:
    explicit CBridge(QObject *parent = 0);

    void updateQMLImage() {emit newFrameAvailable();}

    Q_INVOKABLE void    setMapWidth(qint32 valeur);
    Q_INVOKABLE void    setMapHeight(qint32 valeur);
    Q_INVOKABLE void    resizeMap();
    Q_INVOKABLE void    setMouse(qint32 x, qint32 y);
    Q_INVOKABLE void    setMousePress(qint32 button);
    Q_INVOKABLE void    setMouseRelease(qint32 button);
    Q_INVOKABLE void    setTabIndex(qint32 index);
    Q_INVOKABLE void    save(QString fileName);
    Q_INVOKABLE void    load(QString fileName);
    Q_INVOKABLE void    undo();
    Q_INVOKABLE void    redo();
    Q_INVOKABLE void    copy();
    Q_INVOKABLE void    cut();
    Q_INVOKABLE void    setTool(qint32 tool);
    Q_INVOKABLE void    setSelTileType(qint32 type);
    Q_INVOKABLE void    setSelWallType(qint32 type);

    QStringList tilesList;
    QStringList wallsList;
    CVec2       mMapSize;
    CVec2       mMousePos;
    ETabIndex   mTabIndex;
    EQMLTools   mTool;
    bool        leftPressing;
    bool        leftPressed;
    bool        rightPressed;

    qmlMember(QString,cursorInfo)
    qmlMember(bool,tileSelScrollVis)
    qmlMember(bool,wallSelScrollVis)
    qmlMember(bool,objectSelScrollVis)
    qmlMember(bool,rectSelScrollVis)
    qmlMember(QString,selPosition)
    qmlMember(QString,selSize)
    qmlMember(qint32,selTileType)
    qmlMember(qint32,selWallType)
    qmlMember(qint32,tileType)
    qmlMember(qint32,wallType)

signals:
    Q_SIGNAL void newFrameAvailable();
    void    cursorInfoChanged(QString);
    void    tileSelScrollVisChanged(bool);
    void    wallSelScrollVisChanged(bool);
    void    objectSelScrollVisChanged(bool);
    void    rectSelScrollVisChanged(bool);
    void    selPositionChanged(QString);
    void    selSizeChanged(QString);
    void    selTileTypeChanged(qint32);
    void    selWallTypeChanged(qint32);
    void    tileTypeChanged(qint32);
    void    wallTypeChanged(qint32);
};

//---------------------------------------------------------------------------------------------
class MyImageProvider : public QQuickImageProvider
{
public:
    MyImageProvider();
    QImage requestImage(const QString &id, QSize *size, const QSize &requestedSize);
};

extern CBridge     bridge;

#endif // CBRIDGE_H
