#include "game.h"
#include "walls.h"
#include "player.h"
#include "../common/sources/system/fileCache.h"
#include "../common/sources/system/cgraphics2d.h"
#include <QDebug>
#include "interface.h"

#define SCREEN_WIDTH    320
#define SCREEN_HEIGHT   200

CGame   game;

//-----------------------------------------------------------------------------------------
CGame::CGame()
{
    characters[0].picture = 0;
    characters[0].name = "ZED";
    characters[0].stats[CCharacter::eStatHealth].value = 10;
    characters[0].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[0].stats[CCharacter::eStatStamina].value = 10;
    characters[0].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[0].stats[CCharacter::eStatMana].value = 10;
    characters[0].stats[CCharacter::eStatMana].maxValue = 10;
    characters[0].bodyObjects[CCharacter::eBodyPartRightHand] = -1;

    characters[1].picture = 1;
    characters[1].name = "STAMM";
    characters[1].stats[CCharacter::eStatHealth].value = 5;
    characters[1].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[1].stats[CCharacter::eStatStamina].value = 5;
    characters[1].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[1].stats[CCharacter::eStatMana].value = 5;
    characters[1].stats[CCharacter::eStatMana].maxValue = 10;
    characters[1].bodyObjects[CCharacter::eBodyPartRightHand] = -1;

    characters[2].picture = 2;
    characters[2].name = "WUUF";
    characters[2].stats[CCharacter::eStatHealth].value = 0;
    characters[2].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[2].stats[CCharacter::eStatStamina].value = 10;
    characters[2].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[2].stats[CCharacter::eStatMana].value = 10;
    characters[2].stats[CCharacter::eStatMana].maxValue = 10;
    characters[2].bodyObjects[CCharacter::eBodyPartRightHand] = -1;
}

//-----------------------------------------------------------------------------------------
void CGame::drawWall(QImage* image, CVec2 tablePos, EWallSide side, bool flip)
{
    const char* fileName = walls.getFilename(tablePos, side, flip);

    if (fileName[0] != 0)
    {
        QImage  wallGfx = fileCache.getImage(fileName);
        CVec2 gfxPos = walls.getGfxPos(tablePos, side, flip);

        if (flip == true)
            gfxPos.x -= wallGfx.size().width() - 1;

        graph2D.drawImage(image, gfxPos, wallGfx, 0, flip);
    }
}

//-----------------------------------------------------------------------------------------
QImage CGame::displayMainView()
{
    QImage image(SCREEN_WIDTH,
                 SCREEN_HEIGHT,
                 QImage::Format_RGBA8888);

	// clear the screen
    image.fill(QColor(0, 0, 0));

    if (interface.isInInventory() == false)
    {
        // draw the floor and the ceiling
        QImage  floorGfx = fileCache.getImage("gfx/3DView/Floor.png");
        QImage  ceilingGfx = fileCache.getImage("gfx/3DView/Ceiling.png");

        bool flip = (player.pos.x + player.pos.y + player.dir) & 1;

        graph2D.drawImage(&image, CVec2(0, 33), ceilingGfx, 0, flip);
        graph2D.drawImage(&image, CVec2(0, 72), floorGfx, 0, flip);

        // draw the walls
        CVec2   tablePos;
        CVec2   localPos;
        CVec2   mapPos;
        EWallSide   side;
        CTile*  tile;

        for (int y = 0; y < WALL_TABLE_HEIGHT; ++y)
            for (int x = 0; x < WALL_TABLE_WIDTH; ++x)
            {
                tablePos = walls.getDrawSequence(CVec2(x, y));
                localPos = walls.getMapPos(tablePos);
                mapPos = player.getPosFromLocal(localPos);
                tile = map.getTile(mapPos);

                if (tile != NULL)
                {
                    side = player.getWallSide(eWallSideUp);
                    if (tile->mWalls[side].mType != 0)
                        drawWall(&image, tablePos, eWallSideUp, flip);

                    side = player.getWallSide(eWallSideLeft);
                    if (tile->mWalls[side].mType != 0)
                        drawWall(&image, tablePos, eWallSideLeft, flip);

                    side = player.getWallSide(eWallSideRight);
                    if (tile->mWalls[side].mType != 0)
                        drawWall(&image, tablePos, eWallSideRight, flip);

                    // objects.draw();
                    // monsters.draw();
                }
            }
    }

    interface.display(&image);

    return image;
}
